#include <avr/io.h>
#include <util/delay.h>
#include <stdbool.h>
#include <inttypes.h>
#include "ili9341.h"
#include "ili9341gfx.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <avr/interrupt.h>


#define F_CPU               16000000
#define CENTER 55
#define PAS 50
#define PRESCALER           1024
#define DEBOUNCE_TIME       10
#define WAIT_TIME 1000
#define NUMBER_MAX_NOTES 20


int score;
int keyboard[8] = {PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7};
unsigned int frecv[8] = {262, 294, 330, 349, 392, 440, 494, 303};
int input_by_user[NUMBER_MAX_NOTES];
int sang_by_piano[NUMBER_MAX_NOTES];
int number_notes = 0;
int compute_score = -1;
int round_finished = 0;
int show_score = -1;


extern uint16_t vsetx,vsety,vactualx,vactualy,isetx,isety,iactualx,iactualy;
static FILE mydata = FDEV_SETUP_STREAM(ili9341_putchar_printf, NULL, _FDEV_SETUP_WRITE);

static	long	randx;
static	char	randf;

void
srand(unsigned x)
{
	randx = x;
	randf = 1;
}

rand(void)
{
	if(!randf)
		srand(1);
	return((int)((randx = randx*1103515245L + 12345)>>16) & 077777);
}


void IO_init()
{
    /* Seteaza pinii PB0-7 ca pini de intrare. */
	DDRA = 0;

    /* Activeaza rezistenta de pull-up pentru pinii PB0-7. */
	PORTA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3)
        | (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7);

    /* Seteaza pinii PC0 si PC1, PC3, PC4
      ca pini de intrare */
    // PC0, PC1, PC3, PC4 control

    DDRC &= ~(1 << PC0);
    DDRC &= ~(1 << PC1);
    DDRC &= ~(1 << PC3);
    DDRC &= ~(1 << PC4);
    DDRC |= (1 << PC2);

    /* Activeaza rezistenta de pull-up pentru pinii PC0, PC1 si
    PC2, PC3, PC4. */
    PORTC |= (1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3) | (1 << PC4);

    /* Seteaza pinul PD4 ca pin de iesire. */
    DDRD |= (1 << PD4); // BUZZER

 	 /* Activeaza intreruperi pentru butoanele de control(PC0 si PC1). */
    PCICR |= (1 << PCIE2) | (1 << PCIE3);
    PCMSK2 |= (1 << PCINT16) | (1 << PCINT17);
    PCMSK3 |= (1 << PCINT29);
    PCICR |= (1 << PCIE1);
    PCMSK1 |= (1 << PCINT8) | (1 << PCINT9) | (1 << PCINT10) | (1 << PCINT11)
        | (1 << PCINT12) | (1 << PCINT13) | (1 << PCINT14) | (1 << PCINT15);

}


void timer1_init(void)
{
	TCCR1B |= (1 << WGM12) | (1 << WGM13) | (1 << CS10) | (1 << CS12);
    TCCR1A = 0;

    OCR1B = 0;
    OCR1A = 0;

}


void init(void) {
	stdout = & mydata;
	ili9341_init();//initial driver setup to drive ili9341
	ili9341_setRotation(0);//rotate screen
	ili9341_clear(BLACK);//fill screen with black colour
}



void draw_start(void) {

	ili9341_setcursor(CENTER, CENTER);
	ili9341_settextcolour(PINK, BLACK);
	ili9341_settextsize(3);
	printf("Welcome");

	ili9341_setcursor(CENTER - PAS/2, CENTER + PAS);
	ili9341_settextcolour(PINK, BLACK);
	ili9341_settextsize(3);
	printf("PIANO HERO");

	ili9341_setcursor(CENTER - PAS/2, CENTER + 2 * PAS);
	ili9341_settextcolour(YELLOW, BLACK);
	ili9341_settextsize(3);
	printf("MADALINA");

	ili9341_setcursor(CENTER - PAS/2, CENTER + 3 * PAS);
	ili9341_settextcolour(YELLOW, BLACK);
	ili9341_settextsize(3);
	printf("HURMUZ");


	_delay_ms(10000);
	ili9341_clear(BLACK);//fill screen with black colour
}




/* Intreruperea pentru butoanele de control */
ISR(PCINT2_vect)
{
    //_delay_ms(DEBOUNCE_TIME);

	// PC0 apasat
    if((PINC & (1 << PC0)) == 0)
    {
        number_notes += 5;
    }

    // PC1 apasat
    if((PINC & (1 << PC1)) == 0)
    {
        number_notes -=  1;
    }


}

/*
 * Actualizeaza nota ce trebuie redata.
 */
void update_note(int index)
{
    OCR1A = F_CPU / PRESCALER / frecv[index];
    OCR1B = OCR1A / 2;
    TCNT1 = 0;
}


/*
 * Reda sunetul corespunzator unei note.
 */
void play_note(int index)
{
    if (index < 0 || index > 7)
        return;

    TCCR1A |= (1 << COM1B1) | (1 << WGM10) | (1 << WGM11);
    update_note(index);
}


/*
 * Stinge toate led-urile.
 */
void switch_off_leds(void)
{
    int i;

    for (i = 0; i <= 7; i++)
       PORTA &= ~(1 << keyboard[i]);
}

/*
 * Opreste sunetul.
 */
void end_note()
{
    TCCR1A &= ~(1 << COM1B1);
}

/* Canta o melodie ale carui note sunt alege random. */
void play_song()
{

	int i;
	int r;

	for (i = 0; i < NUMBER_MAX_NOTES; i++) {
		r = rand() % 8;
		play_note(r);
		_delay_ms(4400);
		end_note();
		sang_by_piano[i] = r;
	}
}

/*
 * Intreruperea pentru butoanele de claviatura.
 */
ISR(PCINT1_vect)
{
    int i, ok = 0;


	ili9341_setcursor(CENTER - PAS / 2, CENTER + PAS);
	ili9341_settextcolour(YELLOW, BLACK);
	ili9341_settextsize(3);
	printf("REPLAY SONG");


	ili9341_setcursor(CENTER - PAS / 2, CENTER + 2 * PAS);
	ili9341_settextcolour(YELLOW, BLACK);
	ili9341_settextsize(3);
	printf("SCORE");

	ili9341_setcursor(CENTER - PAS / 2, CENTER + 3 * PAS);
	ili9341_settextcolour(YELLOW, BLACK);
	ili9341_settextsize(3);

	int x = CENTER-PAS/2;
	int y = CENTER + 3 * PAS;


	ili9341_drawvline(0, 0, 320, CYAN);
	ili9341_drawvline(237, 0, 320, CYAN);
	ili9341_drawhline(0, 80, 235, WHITE);
	ili9341_drawhline(0, 200, 235, BLUE);


    _delay_ms(DEBOUNCE_TIME * 3);

	if (round_finished == 0)
		for (i = 0; i < 8; i++) {
			if ((PINA & (1 << keyboard[i])) == 0) {
					PORTA |= (1 << keyboard[i]);
					play_note(i);
					ok = 1;
					input_by_user[number_notes] = i;
					number_notes++;
					if (number_notes == NUMBER_MAX_NOTES) {
						round_finished = 1;
						show_score = 1;
						//printf("DAA");
					}
			}
			if (ok == 0) {
				switch_off_leds();
				end_note();

			}
		}

	_delay_ms(4 * WAIT_TIME);

	if (round_finished == 1 && show_score == 1) {
		for (i = 0; i < NUMBER_MAX_NOTES; i++)
			if (input_by_user[i] != sang_by_piano[i])
				score -= 5;
			else
				score += 100;
		compute_score = -1;
		printf("%d ", score);
		show_score = -1;
		switch_off_leds();
		end_note();
		round_finished = 2;
	}

	_delay_ms(5 * WAIT_TIME);

	if (round_finished == 2) {
		ili9341_setcursor(x, y + 30);
		ili9341_settextcolour(RED, BLACK);
		printf("GAME OVER");
	}

}




int main(void)
{

	 /* Initializeaza intrarile si iesirile. */
    IO_init();

	rand();

	init();
	draw_start();

	/* Activeaza intreruperile. */

	timer1_init();

	play_song();

	_delay_ms(5 * WAIT_TIME);

	/* Activeaza intreruperile. */
    sei();

	while (1) {

	}

	return 0;
}